/*
 *	$Id: acl.cpp 1.7 Sat, 06 Dec 2003 14:45:37 -0800 root $
 *	$ProjectHeader: pptpproxy 1.10 Sat, 06 Dec 2003 14:45:37 -0800 root $
 */

#include <proxy.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/wait.h>
#include <sys/types.h>

void Proxy::addACL(
char *acl
)
{
    if(acl==0)
    {
	fail(
	    true,
	    0,
	    "%s: empty argument for --acl",
	    __func__
	);
    }

    dbg(
	"%s: adding acl %s",
	__func__,
	acl
    );

    char *p= acl= strdup(acl);
    while(p[0]!=0 && p[0]!='/') ++p;
    if(p[0]==0)
    {
	fail(
	    true,
	    0,
	    "%s: %s: incorrect acl syntax",
	    __func__,
	    acl
	);
    }
    p[0]= 0;

    IPAddr snet;
    if(resolve(&snet,acl,false)==false)
    {
	fail(
	    true,
	    0,
	    "%s: couldn't resolve subnet %s",
	    __func__,
	    p+1
	);
    }

    IPAddr mask;
    if(resolve(&mask,p+1,false)==false)
    {
	fail(
	    true,
	    0,
	    "%s: couldn't resolve subnet mask %s",
	    __func__,
	    p+1
	);
    }

    acls.push_back(snet);
    acls.push_back(mask);
    free(acl);
}

void Proxy::addACLCommand(
char *cmd
)
{
    if(cmd==0)
    {
	fail(
	    true,
	    0,
	    "%s: empty argument for --aclCmd",
	    __func__
	);
    }

    dbg(
	"%s: adding aclCmd %s",
	__func__,
	cmd
    );

    aclCmds.push_back(cmd);
}

bool Proxy::checkACL(
IPAddr ip
)
{
    if(isDebugOn())
    {
	dbg(
	    "%s: checking ip %s against acl list",
	    __func__,
	    makeString(ip)
	);
    }

    int n= acls.size();
    for(int i=0;i<n;i+=2)
    {
	IPAddr snet= acls[i+0];
	IPAddr mask= acls[i+1];
	if((ip&mask)==(snet&mask))
	{
	    if(isDebugOn())
	    {
		dbg(
		    "%s: ip %s authorized",
		    __func__,
		    makeString(ip)
		);
	    }
	    return true;
	}
    }

    if(isDebugOn())
    {
	dbg(
	    "%s: checking ip %s against acl cmd list",
	    __func__,
	    makeString(ip)
	);
    }

    n= aclCmds.size();
    for(int j=0;j<n;++j)
    {
	char buffer[8192];
	const char *cmd = aclCmds[j];
	snprintf(buffer, 8190, "%s %s",cmd, makeString(ip));

	if(isDebugOn())
	{
	    dbg(
		"%s: checking ip %s with cmd %s",
		__func__,
		makeString(ip),
		buffer
	    );
	}

	int ret = system(buffer);
	int exitStatus = WEXITSTATUS(ret);

	if(isDebugOn())
	{
	    dbg(
		"%s: command returned %d (IP %s is %s)",
		__func__,
		exitStatus,
		makeString(ip),
		exitStatus==0 ? "ok" : "denied"
	    );
	}
	if(exitStatus==0) return true;
    }

    return false;
}

