/*
 *	$Id: db.cpp 1.6 Fri, 01 Mar 2002 23:46:17 -0800 root $
 *	$ProjectHeader: pptpproxy 1.10 Sat, 06 Dec 2003 14:45:37 -0800 root $
 */

#include <proxy.h>
#include <pthread.h>
#include <arpa/inet.h>

bool Proxy::remapId(
CallId *mappedId,
CallId id
)
{
    if(isDebugOn())
    {
	dbg(
	    "%s: trying to map real id 0x%x for control packet",
	    __func__,
	    id
	);
    }

    pthread_mutex_lock(lock);
	bool success= false;
	int n= links.size();
	for(int i=0;i<n;++i)
	{
	    Link *link= links[i];
	    if(id==link->getFakeCallerId())
	    {
		*mappedId= link->getRealCallerId();
		success= true;
	    }
	    else if(id==link->getRealCallerId())
	    {
		*mappedId= link->getFakeCallerId();
		success= true;
	    }
	    else if(id==link->getFakeCalleeId())
	    {
		*mappedId= link->getRealCalleeId();
		success= true;
	    }
	    else if(id==link->getRealCalleeId())
	    {
		*mappedId= link->getFakeCalleeId();
		success= true;
	    }
	}
    pthread_mutex_unlock(lock);
    return success;
}

bool Proxy::findPeer(
IPAddr	*dst,
CallId	*realCallId,
IPAddr	src,
CallId	fakeCallId
)
{
    if(isDebugOn())
    {
	dbg(
	    "%s: trying to find peer for packet, src=%s fakeCallId=0x%x",
	    __func__,
	    makeString(src),
	    fakeCallId
	);
    }

    pthread_mutex_lock(lock);
	bool success= false;
	int n= links.size();
	for(int i=0;i<n;++i)
	{
	    Link *link= links[i];
	    if(
		src==link->getCalleeIP()		&&
		fakeCallId==link->getFakeCallerId()
	    )
	    {
		*dst= link->getCallerIP();
		*realCallId= link->getRealCallerId();
		if(isDebugOn())
		{
		    char *srcName= strdup(makeString(src));
		    char *dstName= strdup(makeString(*dst));
		    dbg(
			"%s: found peer(caller) for packet, src=%s dst=%s, realId= 0x%x",
			__func__,
			srcName,
			dstName,
			*realCallId
		    );
		    free(srcName);
		    free(dstName);
		}
		success= true;
		break;
	    }
	    else if(
		src==link->getCallerIP()		&&
		fakeCallId==link->getFakeCalleeId()
	    )
	    {
		*dst= link->getCalleeIP();
		*realCallId= link->getRealCalleeId();
		if(isDebugOn())
		{
		    char *srcName= strdup(makeString(src));
		    char *dstName= strdup(makeString(*dst));
		    dbg(
			"%s: found peer(callee) for packet, src=%s dst=%s, realId= 0x%x",
			__func__,
			srcName,
			dstName,
			*realCallId
		    );
		    free(srcName);
		    free(dstName);
		}
		success= true;
		break;
	    }
	}
    pthread_mutex_unlock(lock);

    pthread_mutex_lock(hang);
    pthread_mutex_unlock(hang);

    if(success==false)
    {
	fail(
	    false,
	    0,
	    "%s: discarding unregistered data packet from IP %s, fakeCallId=0x%x",
	    __func__,
	    makeString(src),
	    fakeCallId
	);
    }
    return success;
}

