/*
 *	$Id: log.cpp 1.7 Sat, 06 Dec 2003 14:05:33 -0800 root $
 *	$ProjectHeader: pptpproxy 1.10 Sat, 06 Dec 2003 14:45:37 -0800 root $
 */

#include <proxy.h>
#include <errno.h>
#include <stdio.h>
#include <syslog.h>

void Proxy::vlog(
const char	*type,
const char	*msg,
va_list		arg
)
{
    FILE *f=	daemonized			?
		(logFile?fopen(logFile,"a"):0)	:
		stdout;
    if(f)
    {
	fprintf(f,"pptpproxy: %s",type);
	vfprintf(
	    f,
	    msg,
	    arg
	);
	fputc('\n',f);
	fflush(f);
	if(f!=stdout) fclose(f);
    }
    else
    {
	vsyslog(
	    LOG_ERR,
	    msg,
	    arg
	);
    }
}

void Proxy::log(
const char *type,
const char *msg,
...
)
{
    va_list arg;
    va_start(arg,msg);
	vlog(type,msg,arg);
    va_end(arg);
}

void Proxy::dbg(
const char *msg,
...
)
{
    if(debug)
    {
	va_list arg;
	va_start(arg,msg);
	    vlog("debug  : ",msg,arg);
	va_end(arg);
    }
}


void Proxy::fail(
bool		fatal,
const char	*sys,
const char	*msg,
...
)
{
    const char *err= fatal ? "error  : " : "warning: ";
    if(sys)
    {
	log(
	    err,
	    "%s: %s",
	    sys,
	    strerror(errno)
	);
    }

    if(msg)
    {
	va_list arg;
	va_start(arg,msg);
	    vlog(err,msg,arg);
	va_end(arg);
    }

    if(fatal)
    {
	log(err,"exiting.");
	exit(1);
    }
}

