/*
 *	$Id: options.cpp 1.7 Sat, 06 Dec 2003 14:05:33 -0800 root $
 *	$ProjectHeader: pptpproxy 1.10 Sat, 06 Dec 2003 14:45:37 -0800 root $
 */

#include <proxy.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

static void version(
bool die
)
{
    const char *v= "$ProjectHeader: pptpproxy 1.10 Sat, 06 Dec 2003 14:45:37 -0800 root $\n";
    while(*v && *v!=' ') ++v;
    ++v;

    putchar('\n');
    for(int i=0;i<6;++i)
    {
	while(*v && *v!=' ') putchar(*v++);
	putchar(*v++);
    }

    printf(
	"\n"
	"Written by Emmanuel Mogenet <mgix@mgix.com>\n"
    );
    if(die) exit(0);
}

static void help()
{
    printf(
	"\n"
	"Usage: pptpproxy [options]\n"
	"\n"
	"    Options:\n"
	"\n"
	"        -d, --debug                    Debug mode\n"
	"        -h, --help                     Print this message\n"
	"        -v, --version                  Print version and exit\n"
	"        -n, --nofork                   Do not fork to the background\n"
	"        -l, --log logFile              Log stuff to this file\n"
	"        -a, --acl subnet/mask          Add subnet to access control list.\n"
	"        -x, --aclCmd external command  Launch an external command to verify IP\n"
	"        -p, --proxy [listen[:port],]peer[:port] Specify proxy pair\n"
	"\n"
    );
    exit(0);
}

static void beg()
{
    version(false);
    printf(
	"\n"
	"If you find pptpproxy useful and use it in a professional\n"
	"capacity, please consider making a paypal donation to\n"
	"mgix@mgix.com to help maintain the website.\n"
	"\n"
    );
}

void Proxy::options(
char **argv
)
{
    beg();

    while(1)
    {
	const char *arg= *++argv;
	     if(arg==0)							break;
	else if(0==strcmp(arg,"-h") || 0==strcmp(arg,"--help"))		help();
	else if(0==strcmp(arg,"-v") || 0==strcmp(arg,"--version"))	version(true);
	else if(0==strcmp(arg,"-n") || 0==strcmp(arg,"--nofork"))	noFork=true;
	else if(0==strcmp(arg,"-d") || 0==strcmp(arg,"--debug"))	debug= true;
	else if(0==strcmp(arg,"-l") || 0==strcmp(arg,"--log"))		logFile=(argv ? *++argv : 0);
	else if(0==strcmp(arg,"-a") || 0==strcmp(arg,"--acl"))		addACL(argv ? *++argv : 0);
	else if(0==strcmp(arg,"-x") || 0==strcmp(arg,"--exec"))		addACLCommand(argv ? *++argv : 0);
	else if(0==strcmp(arg,"-p") || 0==strcmp(arg,"--proxy"))	addProxyPair(argv ? *++argv : 0);
	else
	{
	    fail(
		false,
		0,
		"%s: unknown argument %s",
		__func__,
		*argv
	    );
	    help();
	}
    }

    if(isDebugOn()) noFork= true;

    if(
	acls.size()<=0		&&
	aclCmds.size()<=0
    )
    {
	dbg(
	    "%s: no acl specified, adding 0/0",
	    __func__
	);
	addACL("0.0.0.0/0.0.0.0");
    }

    if(pairs.size()<=0)
    {
	fail(
	    true,
	    0,
	    "%s: no valid proxy pair specified",
	    __func__
	);
    }
}

