/*
 *	$Id: pairs.cpp 1.5 Tue, 22 Jan 2002 00:25:01 -0800 root $
 *	$ProjectHeader: pptpproxy 1.10 Sat, 06 Dec 2003 14:45:37 -0800 root $
 */

#include <proxy.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

bool Proxy::parseAddress(
IPAddr		*addr,
TCPPort		*port,
const char	*istr
)
{
    char *p;
    char *str= p= strdup(istr);
    while(p[0]!=0 && p[0]!=':') ++p;

    if(p[0]==0) *port= 1723;
    else
    {
	*(p++)= 0;
	if(1!=sscanf(p,"%d",port))
	{
	    fail(
		false,
		0,
		"%s: address %s:%s has an invalid port",
		__func__,
		str,
		p
	    );
	    free(str);
	    return false;
	}
    }

    *port= htons((uint16_t)*port);
    bool ret= resolve(addr,str,false);
    free(str);
    return ret;
}

void Proxy::addProxyPair(
char *argv
)
{
    if(argv==0)
    {
	fail(
	    true,
	    0,
	    "%s: empty argument for --proxy",
	    __func__
	);
    }

    dbg(
	"%s: adding proxy pair %s",
	__func__,
	argv
    );

    char *p= argv;
    while(p[0]!=0 && p[0]!=',') ++p;

    char *peer;
    char *listen;
    if(p[0]==0)
    {
	listen= strdup("0.0.0.0:1723");
	peer= argv;
    }
    else
    {
	listen= argv;
	peer= p+1;
	p[0]= 0;
    }

    IPAddr listenAddr;
    TCPPort listenPort;
    if(parseAddress(&listenAddr,&listenPort,listen)==false)
    {
	fail(
	    false,
	    0,
	    "%s: specified pair %s -> %s will be ignored because listen address can not be resolved.",
	    __func__,
	    listen,
	    peer
	);
	return;
    }

    IPAddr peerAddr;
    TCPPort peerPort;
    if(parseAddress(&peerAddr,&peerPort,peer)==false)
    {
	fail(
	    false,
	    0,
	    "%s: specified pair %s -> %s will be ignored because peer address can not be resolved.", 
	    __func__,
	    listen,
	    peer
	);
	return;
    }

    int n= pairs.size();
    for(int i=0;i<n;++i)
    {
	IPAddr addr= pairs[i]->getListenAddr();
	TCPPort port= pairs[i]->getListenPort();
	if(
	    listenPort==port	&&
	    (
		addr==listenAddr	||
		listenAddr==0		||
		addr==0
	    )
	)
	{
	    fail(
		false,
		0,
		"%s: pair %s -> %s will be ignored because it conflicts with previously specified pair %s -> %s",
		__func__,
		listen,
		peer,
		pairs[i]->getListenName(),
		pairs[i]->getPeerName()
	    );
	    return;
	}
    }

    Pair *pair= new Pair(
	this,
	listen,
	listenAddr,
	listenPort,
	peer,
	peerAddr,
	peerPort
    );

    if(pair->getSocket()<0) delete pair;
    else pairs.push_back(pair);
}

Proxy::Pair::Pair(
Proxy		*_proxy,
const char	*_listenStr,
IPAddr		_listenAddr,
TCPPort		_listenPort,
const char	*_peerStr,
IPAddr		_peerAddr,
TCPPort		_peerPort
)
{
    socket= -1;
    proxy= _proxy;

    listenStr= _listenStr;
    listenAddr= _listenAddr;
    listenPort= _listenPort;

    peerStr= _peerStr;
    peerAddr= _peerAddr;
    peerPort= _peerPort;

    int s= proxy->makeSocket(SOCK_STREAM,0,false);
    if(s<0)
    {
	proxy->fail(
	    false,
	    0,
	    "%s: couldn't make socket to listen on %s",
	    __func__,
	    listenStr
	);
	return;
    }

    if(proxy->setNonBlocking(s,true,false)==false)
    {
	proxy->fail(
	    false,
	    0,
	    "%s: couldn't set non blocking flag on listen socket %s",
	    __func__,
	    listenStr
	);
	close(s);
	return;
    }

    struct sockaddr_in addr;
    memset(&addr,0,sizeof(addr));
    addr.sin_family= AF_INET;
    addr.sin_port= (uint16_t)listenPort;
    addr.sin_addr.s_addr= listenAddr;
    if(bind(s,(struct sockaddr*)&addr,sizeof(addr))<0)
    {
	proxy->fail(
	    false,
	    "bind",
	    "%s: couldn't bind to %s",
	    __func__,
	    listenStr
	);
	close(s);
	return;
    }

    if(listen(s,100)<0)
    {
	proxy->fail(
	    false,
	    "listen",
	    "%s: couldn't listen on %s",
	    __func__,
	    listenStr
	);
	close(s);
	return;
    }


    socket= s;
}

Proxy::Pair::~Pair()
{
    close(socket);
}

