/*
 *	$Id: proxy.h 1.12 Sat, 06 Dec 2003 14:05:33 -0800 root $
 *	$ProjectHeader: pptpproxy 1.10 Sat, 06 Dec 2003 14:45:37 -0800 root $
 */

#include <vector>
#include <stdint.h>
#include <stdarg.h>
#include <netinet/in.h>

#ifndef linux
typedef int socklen_t;
#endif

class Proxy
{
public:
		typedef uint32_t	IPAddr;
		typedef uint32_t	TCPPort;
		typedef uint32_t	CallId;

		class Pair
		{
		private:
			int		socket;
			Proxy		*proxy;

			IPAddr		listenAddr;
			TCPPort		listenPort;
			const char	*listenStr;

			IPAddr		peerAddr;
			TCPPort		peerPort;
			const char	*peerStr;
		public:
		    Pair(
			Proxy		*_proxy,
			const char	*_listenStr,
			IPAddr		_listenAddr,
			TCPPort		_listenPort,
			const char	*_peerStr,
			IPAddr		_peerAddr,
			TCPPort		_peerPort
		    );
		    ~Pair();

		    int getSocket()		{ return socket;		}
		    IPAddr getListenAddr()	{ return listenAddr;		}
		    TCPPort getListenPort()	{ return listenPort;		}
		    IPAddr getPeerAddr()	{ return peerAddr;		}
		    TCPPort getPeerPort()	{ return peerPort;		}
		    const char *getPeerName()	{ return peerStr;		}
		    const char *getListenName()	{ return listenStr;		}
		};

		class Link
		{
		private:
			Pair		*pair;
			Proxy		*proxy;
			int		calleeSocket;
			int		callerSocket;

			IPAddr		callerIP;
			CallId		realCallerId;
			CallId		fakeCallerId;

			IPAddr		calleeIP;
			CallId		realCalleeId;
			CallId		fakeCalleeId;
		public:
		    Link(
			Proxy	*_proxy,
			int	pairIndex
		    );
		    ~Link();

		    bool tcpPacket(bool callerPacket);

		    Pair *getPair()		{ return pair;			}
		    int getCalleeSocket()	{ return calleeSocket;		}
		    int getCallerSocket()	{ return callerSocket;		}
		    const char *getPeerName()	{ return pair->getPeerName();	}
		    const char *getListenName()	{ return pair->getListenName();	}

		    CallId getRealCallerId()	{ return realCallerId;		}
		    CallId getFakeCallerId()	{ return fakeCallerId;		}
		    IPAddr getCallerIP()	{ return callerIP;		}

		    CallId getRealCalleeId()	{ return realCalleeId;		}
		    CallId getFakeCalleeId()	{ return fakeCalleeId;		}
		    IPAddr getCalleeIP()	{ return calleeIP;		}
		};

		bool			debug;
		bool			noFork;
		bool			daemonized;
		int			greSocket;
		const char		*logFile;
		std::vector<Pair*>	pairs;
		std::vector<Link*>	links;
		std::vector<IPAddr>	acls;
		std::vector<char*>	aclCmds;
		CallId			calleeIdPool;
		CallId			callerIdPool;

		pthread_mutex_t *lock;
		pthread_mutex_t *hang;

		void	addACL(char *acl);
		bool	checkACL(IPAddr ip);
		void	addACLCommand(char *acl);

		bool	remapId(CallId*,CallId);
		void	addProxyPair(char *acl);
		bool	findPeer(IPAddr*,CallId*,IPAddr,CallId);

		void	dbg(const char *msg= 0,...);
		bool	isDebugOn() { return debug; }
		void	log(const char *type,const char *msg= 0,...);
		void	vlog(const char *type,const char *msg,va_list);
		void	fail(bool fatal,const char *sys,const char *msg= 0,...);

		CallId	allocCalleeId();
		CallId	allocCallerId();

		void	thread();
		void	startThread();
    static	void	*threadHead(void*);

		void	server();
		void	daemonize();
	  const char	*makeString(IPAddr);
		void	options(char **argv);
		int	makeSocket(int type,int proto,bool fatal);
		bool	setNonBlocking(int s,bool yes,bool fatal);
		bool	resolve(IPAddr*,const char *add,bool fatal);
		bool	parseAddress(IPAddr*,TCPPort*,const char*);

public:
		Proxy(char **);
		~Proxy();
};

