/*
 *	$Id: server.cpp 1.9 Sat, 06 Dec 2003 14:05:33 -0800 root $
 *	$ProjectHeader: pptpproxy 1.10 Sat, 06 Dec 2003 14:45:37 -0800 root $
 */

#include <proxy.h>
#include <errno.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/types.h>

static inline void addSocket(
int	socket,
int	*max,
fd_set	*readSet,
fd_set	*exceptionSet
)
{
    if(socket>*max) *max= socket;
    FD_SET(socket,readSet);
    FD_SET(socket,exceptionSet);
}

void Proxy::server()
{
    dbg("starting tcp proxy server thread");

    fd_set readSet;
    fd_set exceptionSet;
    std::vector<int> deadLinks;
    std::vector<Link*> newLinks;
    while(1)
    {
	int i;
	int n;
	int max= -1;
	FD_ZERO(&readSet);
	FD_ZERO(&exceptionSet);

	n= pairs.size();
	for(i=0;i<n;++i)
	{
	    addSocket(
		pairs[i]->getSocket(),
		&max,
		&readSet,
		&exceptionSet
	    );
	}

	n= links.size();
	for(i=0;i<n;++i)
	{
	    addSocket(
		links[i]->getCallerSocket(),
		&max,
		&readSet,
		&exceptionSet
	    );
	    addSocket(
		links[i]->getCalleeSocket(),
		&max,
		&readSet,
		&exceptionSet
	    );
	}

	int ret= select(1+max,&readSet,0,&exceptionSet,0);
	if(ret<0 && errno!=EINTR && errno!=EAGAIN)
	{
	    fail(
		false,
		"select",
		"%s: select failed !\n",
		__func__
	    );
	}
	else if(ret>0)
	{
	    n= pairs.size();
	    for(i=0;i<n;++i)
	    {
		int s= pairs[i]->getSocket();
		if(FD_ISSET(s,&readSet))
		{
		    if(isDebugOn())
		    {
			dbg(
			    "%s: establishing new proxy link from %s to %s",
			    __func__,
			    pairs[i]->getListenName(),
			    pairs[i]->getPeerName()
			);
		    }

		    bool newLinkOK = true;
		    Link *newLink = new Link(this,i);
		    newLinkOK = newLinkOK && newLink->getCallerSocket()>=0;
		    newLinkOK = newLinkOK && newLink->getCalleeSocket()>=0;

		    if(isDebugOn())
		    {
			dbg(
			    "%s: newLink(%s -> %s) %s.",
			    __func__,
			    pairs[i]->getListenName(),
			    pairs[i]->getPeerName(),
			    newLinkOK ? "suceeded" : "failed"
			);
		    }
		    if(newLinkOK) newLinks.push_back(new Link(this,i));
		    else          delete newLink;
		}
		if(FD_ISSET(s,&exceptionSet))
		{
		    fail(
			false,
			"select",
			"%s: exception on listen socket %s -> %s!\n",
			__func__,
			pairs[i]->getListenName(),
			pairs[i]->getPeerName()
		    );
		}
	    }

	    i= links.size();
	    while(i--)
	    {
		bool linkOK = true;
		int s1= links[i]->getCallerSocket();
		int s2= links[i]->getCalleeSocket();
		if(FD_ISSET(s1,&readSet)) linkOK = linkOK && links[i]->tcpPacket(true);
		if(FD_ISSET(s2,&readSet)) linkOK = linkOK && links[i]->tcpPacket(false);
		if(FD_ISSET(s1,&exceptionSet)) linkOK= false;
		if(FD_ISSET(s2,&exceptionSet)) linkOK= false;

		if(linkOK==false)
		{
		    fail(
			false,
			"select",
			"%s: exception on tcp link %s -> %s. Terminating link.\n",
			__func__,
			links[i]->getListenName(),
			links[i]->getPeerName()
		    );
		    deadLinks.push_back(i);
		}
	    }

	    int n1= newLinks.size();
	    int n2= deadLinks.size();
	    if(n1 || n2)
	    {
		pthread_mutex_lock(hang);
		    pthread_mutex_lock(lock);
			while(n1--) links.push_back(newLinks[n1]);
			while(n2--)
			{
			    int i= deadLinks[n2];
			    dbg(
				"%s: removing links[%d]= %s->%s",
				__func__,
				i,
				links[i]->getListenName(),
				links[i]->getPeerName()
			    );

			    Link *link= links[i];
			    links[i]=links[links.size()-1];
			    links.pop_back();
			    delete link;
			}
		    pthread_mutex_unlock(lock);
		pthread_mutex_unlock(hang);
		deadLinks.clear();
		newLinks.clear();
	    }
	}
    }
}

