/*
 *	$Id: thread.cpp 1.8 Mon, 01 Jul 2002 17:44:29 -0700 root $
 *	$ProjectHeader: pptpproxy 1.10 Sat, 06 Dec 2003 14:45:37 -0800 root $
 */

#include <proxy.h>
#include <errno.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include <arpa/inet.h>
#include <stdio.h>

void Proxy::thread()
{
    dbg("starting gre proxy server thread");

    uint8_t buf[16384];
    while(1)
    {
	struct sockaddr_in from;
	socklen_t fromLen= sizeof(from);
	int n= recvfrom(
	    greSocket,
	    buf,
	    16380,
	    0,
	    (struct sockaddr*)&from,
	    &fromLen
	);

	if(n<0)
	{
	    if(errno==EINTR)		continue;
	    else if(errno==EAGAIN)	continue;
	    else
	    {
		fail(	
		    false,
		    "recvfrom",
		    "%s:recvfrom failed on gre socket",
		    __func__
		);
		continue;
	    }
	}

	uint8_t *packet= buf;
	if((buf[0]&0xF0)==0x40)
	{
	    int headerSize= (buf[0]&0x0F)*4;
	    packet+= headerSize;
	    n-= headerSize;
	}

	uint32_t dst;
	uint32_t src= from.sin_addr.s_addr;
	uint32_t fakeCallId= packet[6] | (((uint32_t)packet[7])<<8);

	if(isDebugOn())
	{
	    dbg(
		"%s: receiving gre data packet from %s, fakeCallId=0x%x",
		__func__,
		inet_ntoa(from.sin_addr),
		fakeCallId
	    );
	}

	CallId realCallId;
	if(findPeer(&dst,&realCallId,src,fakeCallId))
	{
	    struct sockaddr_in to;
	    socklen_t toLen= sizeof(to);
	    to.sin_addr.s_addr= dst;

	    packet[6]= (realCallId>>0)&0xFF;
	    packet[7]= (realCallId>>8)&0xFF;

	    if(isDebugOn())
	    {
		dbg(
		    "%s: forwarding gre data packet to %s",
		    __func__,
		    inet_ntoa(to.sin_addr)
		);
	    }

	    uint8_t *p= packet;
	    while(n>0)
	    {
		int s= sendto(
		    greSocket,
		    p,
		    n,
		    0,
		    (const sockaddr*)&to,
		    toLen
		);
		if(s>=0)
		{
		    p+= s;
		    n-= s;
		}
		else
		{
		    if(errno==EINTR)		continue;
		    else if(errno==EAGAIN)	continue;
		    else
		    {
			fail(	
			    false,
			    "sendto",
			    "%s:sendto failed on gre socket",
			    __func__
			);
		    }
		}
	    }
	}
    }
}

void *Proxy::threadHead(
void *vp
)
{
    Proxy *proxy= (Proxy*)vp;
    proxy->thread();
    return 0;
}

void Proxy::startThread()
{
    int s= makeSocket(SOCK_RAW,47,true);

#ifdef SOL_IP
    int solip= SOL_IP;
#else
    int solip= 0;
#endif

    int opt= 0;
    if(setsockopt(s,solip,IP_HDRINCL,&opt,sizeof(opt))<0)
    {
	fail(
	    true,
	    "setsockopt",
	    "%s:Couldn't do a setsockopt(SOL_IPIP_HDRINCL) on GRE socket",
	    __func__
	);
    }

    greSocket= s;
    setNonBlocking(s,false,true);

    pthread_t thread;
    if(pthread_create(&thread,0,threadHead,this)<0)
    {
	fail(
	    true,
	    "pthread_create",
	    "%s:couldn't start GRE thread",
	    __func__
	);
    }
}

