/*
 *	$Id: utils.cpp 1.6 Mon, 01 Jul 2002 17:44:29 -0700 root $
 *	$ProjectHeader: pptpproxy 1.10 Sat, 06 Dec 2003 14:45:37 -0800 root $
 */

#include <proxy.h>
#include <netdb.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>


bool Proxy::resolve(
IPAddr		*result,
const char	*address,
bool		fatal
)
{
    if(0==strcmp(address,"0.0.0.0"))
    {
	*result= 0;
	return true;
    }

    struct hostent *he= gethostbyname(address);
    if(he==0)
    {
	fail(
	    fatal,
	    "gethostbyname",
	    "%s: can't resolve IP for %s",
	    __func__,
	    address
	);
	return false;
    }
    *result= *(uint32_t*)&(he->h_addr)[0];
    return true;
}

int Proxy::makeSocket(
int	type,
int	proto,
bool	fatal
)
{
    int s= socket(AF_INET,type,proto);
    if(s<0)
    {
	fail(
	    fatal,
	    "socket",
	    "%s: couldn't create socket%s",
	    __func__,
	    errno==EPERM ? " (Permission denied). Try to run this as root" : ""
	);
    }

    int opt= 1;
    if(setsockopt(s,SOL_SOCKET,SO_REUSEADDR,&opt,sizeof(opt))<0)
    {
	fail(
	    fatal,
	    "setsockopt",
	    "%s: couldn't do a setsockopt(SO_REUSEADDR) on socket",
	    __func__
	);
    }

    struct linger lng;
    memset(&lng,0,sizeof(lng));
    if(setsockopt(s,SOL_SOCKET,SO_LINGER,&lng,sizeof(lng))<0)
    {
	fail(
	    fatal,
	    "setsockopt",
	    "%s: couldn't do a setsockopt(SO_LINGER) on socket",
	    __func__
	);
    }

    return s;
}

void Proxy::daemonize()
{
    if(debug==false)
    {
	daemonized= true;
	if(signal(SIGHUP ,SIG_IGN)<0) fail(true,"signal","%s: SIGHUP",__func__);
	if(signal(SIGALRM,SIG_IGN)<0) fail(true,"signal","%s: SIGALRM",__func__);
	if(signal(SIGPIPE,SIG_IGN)<0) fail(true,"signal","%s: SIGPIPE",__func__);
	if(signal(SIGUSR1,SIG_IGN)<0) fail(true,"signal","%s: SIGUSR1",__func__);
	if(signal(SIGUSR2,SIG_IGN)<0) fail(true,"signal","%s: SIGUSR2",__func__);
	if(chdir("/")<0) fail(true,"chdir","%s: chdir(/)",__func__);
	umask(0);
	close(0);
	close(1);
	close(2);

	openlog(	
	    "pptpproxy",
	    LOG_PID	|
	    LOG_NDELAY,
	    LOG_DAEMON
	);

	int childPid= fork();
	if(childPid<0) fail(true,"fork","%s: fork failed",__func__);
	else if(childPid)
	{
	    while(1)
	    {
		int status;
		int child= waitpid(-1,&status,WNOHANG);
		if(child==-1 && errno!=EINTR && errno!=EAGAIN) fail(true,"waitpid","%s: couldn't double-fork",__func__);
		if(child==childPid) exit(0);
	    }
	}

	childPid= fork();
	if(childPid<0) fail(true,"fork","%s: couldn't double-fork",__func__);
	else if(childPid) exit(0);
	setsid();
    }
}

bool Proxy::setNonBlocking(
int	s,
bool	yes,
bool	fatal
)
{
    int flags= fcntl(s,F_GETFL,0);
    if(flags<0)
    {
	fail(
	    fatal,
	    "fcntl",
	    "%s: couldn't get socket flags",
	    __func__
	);
	return false;
    }

    flags= yes ? (flags|O_NONBLOCK) : (flags&~O_NONBLOCK);
    if(fcntl(s,F_SETFL,flags)<0)
    {
	fail(
	    fatal,
	    "fcntl",
	    "%s: couldn't change O_NONBLOCK on socket",
	    __func__
	);
	return false;
    }

    return true;
}

const char *Proxy::makeString(
IPAddr ip
)
{
    struct sockaddr_in addr;
    addr.sin_addr.s_addr= ip;
    return inet_ntoa(addr.sin_addr);
}

